// cacheManager.js


export function loadLocalVersionAndApplyFilters(applyFilters) {
    chrome.storage.local.get(['needlesData', 'keywordsData'], function (data) {
        // Fetch the versions from the cached data
        const cachedNeedlesVersion = data.needlesData ? data.needlesData.meta.version : 0;
        const cachedKeywordsVersion = data.keywordsData ? data.keywordsData.meta.version : 0;

        // Load the versions from the local files
        loadLocalVersions((localNeedlesVersion, localKeywordsVersion) => {
            if (cachedNeedlesVersion >= localNeedlesVersion && cachedKeywordsVersion >= localKeywordsVersion) {
                console.log('Using cached data (local cache is up to date)');
                applyFilters(data.needlesData, data.keywordsData);
            } else {
                console.log('Local files have been updated. Reloading and caching new versions...');
                loadPackagedFilesAndApply(applyFilters);
            }
        });
    });
}

// Helper function to load versions from local files
function loadLocalVersions(callback) {
    fetch(chrome.runtime.getURL('rules/needles.json'))
        .then(response => response.json())
        .then(needlesData => {
            fetch(chrome.runtime.getURL('rules/keywords.json'))
                .then(response => response.json())
                .then(keywordsData => {
                    const localNeedlesVersion = needlesData.meta.version || 0;
                    const localKeywordsVersion = keywordsData.meta.version || 0;
                    callback(localNeedlesVersion, localKeywordsVersion);
                })
                .catch(err => {
                    console.error('Error loading keywords.json:', err);
                    callback(0, 0);
                });
        })
        .catch(err => {
            console.error('Error loading needles.json:', err);
            callback(0, 0);
        });
}

export function loadPackagedFilesAndApply(applyFilters) {
    fetch(chrome.runtime.getURL('rules/needles.json'))
        .then(response => response.json())
        .then(needlesData => {
            fetch(chrome.runtime.getURL('rules/keywords.json'))
                .then(response => response.json())
                .then(keywordsData => {
                    chrome.storage.local.set({ needlesData, keywordsData }, () => {
                        console.log('Packaged files cached locally.');
                        applyFilters(needlesData, keywordsData);
                    });
                })
                .catch(err => console.error('Error loading keywords.json:', err));
        })
        .catch(err => console.error('Error loading needles.json:', err));
}