export function filterOrReplaceContent(elements, needlesData, filterType, keywordsData) {
    const REPLACEMENT_URL = "http://www.yellowheart.net/goto";  // Base replacement URL

    function escapeRegExp(string) {
        return string.replace(/[.*+\-?^${}()|[\]\\]/g, '\\$&');
    }

    function getKeywordMatches(keywordList) {
        const bodyText = document.body.innerText.toLowerCase();
        let keywordMatches = [];

        Object.keys(keywordList).forEach(keywordKey => {
            const keywords = keywordList[keywordKey].kw.split(',').map(kw => kw.trim());
            const keywordCount = keywords.reduce((count, kw) => {
                const regex = new RegExp(kw, 'gi');
                const matchCount = (bodyText.match(regex) || []).length;
                return count + matchCount;
            }, 0);

            if (keywordCount > 0) {
                keywordMatches.push({
                    key: keywordKey,
                    txt: keywordList[keywordKey].txt,
                    count: keywordCount
                });
            }
        });

        // Sort by the number of matches
        console.log("Keyword matches found: ", keywordMatches);
        keywordMatches.sort((a, b) => b.count - a.count);
        return keywordMatches;
    }

    const keywordList = keywordsData.data.keywords;  // Access the data property
    const keywordMatches = getKeywordMatches(keywordList);

    chrome.storage.sync.get(['turnEmoji', 'goldEmoji', 'selectedNeedleGroups'], function (emojiData) {
        const turnEmoji = emojiData.turnEmoji || "👉";
        const goldEmoji = emojiData.goldEmoji || "💛";
        const selectedNeedleGroups = emojiData.selectedNeedleGroups || ['demo'];  // Default to ['demo']

        console.log("Using emojis:", turnEmoji, goldEmoji);
        console.log("Selected needle groups:", selectedNeedleGroups);

        // Collect needles only from selected needle groups
        const needlesToApply = [];
        const needleGroups = needlesData.data.needleGroups;  // Access the data property

        selectedNeedleGroups.forEach(groupKey => {
            const needleGroup = needleGroups[groupKey];
            if (needleGroup && needleGroup.needles) {
                needleGroup.needles.forEach(needle => {
                    needlesToApply.push({
                        needle: needle,
                        emoji: needleGroup.emoji,
                        groupId: groupKey  // Add the group ID for appending to the URL
                    });
                });
            } else {
                console.warn('Selected needle group not found:', groupKey);
            }
        });

        console.log("Needles to apply:", needlesToApply);

        // Initialize keyword index for cycling through keyword matches
        let keywordIndex = { index: 0 };

        elements.forEach(tag => {
            const foundElements = document.getElementsByTagName(tag);
            console.log(`Found ${foundElements.length} <${tag}> elements`);

            for (let element of foundElements) {
                if (tag === 'a') {
                    handleLinkReplacement(element, needlesToApply, filterType, turnEmoji, goldEmoji, keywordMatches, keywordIndex);
                } else {
                    // Process text nodes (non-links)
                    for (let i = 0; i < element.childNodes.length; i++) {
                        const child = element.childNodes[i];
                        if (child.nodeType === 3) { // Text node
                            let textContent = child.nodeValue;
                            console.log(`Original text content: ${textContent}`);

                            let newContent = document.createDocumentFragment();
                            let lastIndex = 0;
                            let needlesMatched = false;

                            needlesToApply.forEach(needleItem => {
                                const escapedNeedle = escapeRegExp(needleItem.needle);
                                const needleRegex = new RegExp(`(^|[^\\p{L}\\p{N}_])(${escapedNeedle})(?=[^\\p{L}\\p{N}_]|$)`, 'giu');
                                let match;
                                while ((match = needleRegex.exec(textContent)) !== null) {
                                    needlesMatched = true;
                                    const matchStart = match.index + match[1].length;
                                    const matchEnd = needleRegex.lastIndex;

                                    // Add text before the match
                                    if (matchStart > lastIndex) {
                                        const textNode = document.createTextNode(textContent.substring(lastIndex, matchStart));
                                        newContent.appendChild(textNode);
                                    }

                                    // Get the keyword match for this needle
                                    let keywordMatch = null;
                                    if (keywordMatches.length > 0) {
                                        keywordMatch = keywordMatches[keywordIndex.index % keywordMatches.length];
                                        keywordIndex.index++;  // Increment the index for the next replacement
                                    }

                                    const keywordText = keywordMatch ? keywordMatch.txt : "";
                                    const keywordKey = keywordMatch ? keywordMatch.key : "default";

                                    // Create the replacement link
                                    const replacementText = `[${needleItem.emoji} ${turnEmoji} ${goldEmoji}${keywordText ? ' ' + keywordText : ''}]`;
                                    const href = `${REPLACEMENT_URL}?group=${needleItem.groupId}&kw=${keywordKey}`;
                                    const linkElement = document.createElement('a');
                                    linkElement.href = href;
                                    linkElement.target = '_blank';
                                    linkElement.textContent = replacementText;
                                    newContent.appendChild(linkElement);

                                    lastIndex = matchEnd;
                                }
                            });

                            if (needlesMatched) {
                                // Add remaining text after the last match
                                if (lastIndex < textContent.length) {
                                    const textNode = document.createTextNode(textContent.substring(lastIndex));
                                    newContent.appendChild(textNode);
                                }

                                // Replace the original text node with the new content
                                element.replaceChild(newContent, child);
                            }
                        }
                    }
                }
            }
        });
    });

    function handleLinkReplacement(linkElement, needlesToApply, filterType, turnEmoji, goldEmoji, keywordMatches, keywordIndex) {
        let linkText = linkElement.textContent;
        console.log(`Original link text: ${linkText}`);

        let needleMatched = null; // To keep track of the matching needle

        // Loop through needles and stop at the first match
        for (let i = 0; i < needlesToApply.length; i++) {
            const needleItem = needlesToApply[i];
            const escapedNeedle = escapeRegExp(needleItem.needle);
            const needleRegex = new RegExp(`(^|[^\\p{L}\\p{N}_])(${escapedNeedle})(?=[^\\p{L}\\p{N}_]|$)`, 'giu');
            if (needleRegex.test(linkText)) {
                console.log(`Needle match found for "${needleItem.needle}" in link text: ${linkText}`);
                needleMatched = needleItem; // Save the matching needle
                break; // Exit the loop after the first match
            }
        }

        if (needleMatched && filterType === 'replace') {
            console.log(`Replacing link text with emojis`);

            // Get the keyword match for this needle
            let keywordMatch = null;
            if (keywordMatches.length > 0) {
                keywordMatch = keywordMatches[keywordIndex.index % keywordMatches.length];
                keywordIndex.index++;  // Increment the index for the next replacement
            }

            const keywordText = keywordMatch ? keywordMatch.txt : "";
            const keywordKey = keywordMatch ? keywordMatch.key : "default";

            // Perform replacement with emojis and append text from matched keywords
            const replacementText = `[turn ${needleMatched.emoji} ${turnEmoji} ${goldEmoji}${keywordText ? ' with ' + keywordText : ''}]`;
            linkElement.textContent = replacementText;  // Update link text

            // Update the href to the REPLACEMENT_URL and append the needle group ID and keyword key
            linkElement.href = `${REPLACEMENT_URL}/${keywordKey}/${needleMatched.groupId}&kw=`;
            linkElement.target = '_blank';  // Open in a new tab
            console.log(`Updated link href: ${linkElement.href}`);
        }
    }
}
